/**********************************************************************
 *	PS2 Keyboard Library
 **********************************************************************
 *
void  kbd_init(void);			// : 
uchar kbd_getchar(void);		// : ASCII R[h擾.
uchar kbd_peekcode(void);		// : FIFOL[TOP.
uchar kbd_getcode(void);		// : FIFOL[̎o.
uchar kbd_getpress(uchar c);	// : A^C CӃL[̉ ̎擾.

 */
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>

#include "config.h"
#include "task.h"
#include "ps2keyb.h"
#include "sound.h"

#if	PS2KBD_USE_GETCHAR
#include "keycode.h"
#endif

#include "keyname.h"


#if	PS2KBD_USE_INTERRUPT
//  use PCINT8 hardhandler
#define	INTERRUPT_SERVICE(vect)		ISR(vect)
#else
//	use TIMER1 softhandler
#define	INTERRUPT_SERVICE(vect)		void 	kbd_handler(void)
#endif

/**********************************************************************
 *	FIFÕTCY.
 **********************************************************************
 */
#define	BUFF_SIZE 		32			//Qׂ̂.
#define	BUFF_SIZE_MASK	(BUFF_SIZE-1)

#define	PREFIX_E0		0x1
#define	PREFIX_E1		0x2			// [Pause]=E1 14,77,E1 14,F0 77
#define	PREFIX_F0		0x4

#define	KBD_SHIFT		0x1



/**********************************************************************
 *	[NGA.
 **********************************************************************
 */
static	uchar bitcount;				//	0..10 : start=0,data[8],oddparity,stop=1
static	uchar inptr, outptr;		// FIFO ̎oA݃|C^.
static	uchar kbd_buffer[BUFF_SIZE];// FIFO.
static  uchar kbd_data;				// Holds the received	scan code
static  uchar kbd_shift;			// Holds shift , ctrl , alt status
static  uchar prefix;				// Holds last scan code prefix

#if	PS2KBD_USE_PRESSTABLE	
static	uchar kbd_stat[32];			// 128̃L[̉ (bit table).
#endif
/**********************************************************************
 *	ps2kbd : 
 **********************************************************************
 */
void kbd_init(void)
{
#if	PS2KBD_USE_INTERRUPT		// use PCINT8
	//|[g̏.
	PCICR  = 1<<1;	// PCINT1 (8..14) Enable.
	PCMSK1 = 1<<0;	// PCINT8 Mask off.
#endif

	//ϐ̏.
	inptr  =	0;
	outptr =	0;
	bitcount = 11;
	prefix =	0;
	kbd_shift =	0;
}

#if	PS2KBD_USE_PRESSTABLE	
/**********************************************************************
 *	ps2kbd : 128̃L[̉mask bit.
 **********************************************************************
 */
static uchar get_press_mask(uchar c)
{
	return 1 << (c & 7);
}
static uchar get_press_idx(uchar c)
{
	return (c >> 3);
}

/**********************************************************************
 *	ps2kbd : 128̃L[̉ ̃NA.
 **********************************************************************
 */
void clr_press_table(void)
{
	uchar i;
	for(i=0;i<16;i++) {
		kbd_stat[i] = 0;
	}
}
/**********************************************************************
 *	ps2kbd : 128̃L[̉ ̍XV.
 **********************************************************************
 */
static void set_press_table(uchar c,uchar pre)
{
	uchar idx  = get_press_idx(c);
	uchar mask = get_press_mask(c);
	if(pre & PREFIX_F0) {	//ꂽ,gV[PX͖.
		kbd_stat[idx] &= ~mask;
	}else{
		kbd_stat[idx] |=  mask;
	}
}

/**********************************************************************
 *	ps2kbd : 128̃L[̉ ̎擾.
 **********************************************************************
 *	  c : XLR[h.
 *	߂l  : [̏ꍇÃXLR[h̃L[́uꂽԁvɂ.
 *			  [̏ꍇÃXLR[h̃L[́uꂽԁvɂ.
 */
uchar kbd_getpress(uchar c)
{
	uchar idx  = get_press_idx(c);
	uchar mask = get_press_mask(c);
	return kbd_stat[idx] & mask;
}
#else

#endif
/**********************************************************************
 *	ps2kbd : FIFOL[̑.
 **********************************************************************
 */
static	void put_que(uchar c)
{
	uchar p = inptr;

	if(c==0) return;			// 0 FIFOɏ܂Ȃ.

	kbd_buffer[p]=c;
	p++; p &= BUFF_SIZE_MASK;	//  p i߂Ă݂.

	if(p != outptr) {	//  p o|C^ɒǂĂȂꍇ̂,
		inptr = p;		// |C^XV.
	}
}

/**********************************************************************
 *	ps2kbd : FIFOL[TOP.
 **********************************************************************
 */
uchar kbd_peekcode(void)
{
	uchar p = outptr;
	if(p == inptr) {			//L[Ɣf.
		return 0;				//͂Ȃ.
	}
	return kbd_buffer[p];		//L[TOP邾.
}

/**********************************************************************
 *	ps2kbd : FIFOL[̎o.
 **********************************************************************
 *	L[͂ȂƂ́A0Ԃ.
 *	L[͂Ƃ́AXLR[h([)1oCgԂ.
 *ӁF
 *	ASCIIR[ĥ݂擾ꍇ́Akbd_getcode()،Ă΂ɁA
 *	kbd_getchar()̂قgpB
 *	݂ČĂяoꍇAkbd_getchar()̓삪ȂB
 */
uchar kbd_getcode(void)
{
	uchar c;
	uchar p = outptr;
	if(p == inptr) {			//L[Ɣf.
		return 0;				//͂Ȃ.
	}
	c = kbd_buffer[p];			//L[o.
	p++; p &= BUFF_SIZE_MASK;	//  p i߂Ă݂.
	outptr = p;
	return c;
}

#if	PS2KBD_USE_GETCHAR
/**********************************************************************
 *	ps2kbd : getchar ASCII R[h擾.
 **********************************************************************
 *	L[͂ȂƂ́A0Ԃ.
 *	L[u[N(ꂽ)OuR[h(0xF0)gV[PX(0xE0)
 *	擾ꍇ́A prefixƂX^eBbNϐɂ̃tO
 *	cA߂l 0 ɂȂ.ĩR[h擾ɃtOQƂj
 */
uchar kbd_getchar(void)
{
	uchar pre = prefix;			//ÕR[hgR[hĂ?
	uchar c = kbd_getcode();
	uchar idx;

	if(c==0) return c;			//͖.

	// QoCgȏ̃V[PXꍇ́AOuR[hŏ:
	if(c == 0xe0) {				//gV[PX E0.
		prefix |= PREFIX_E0;
		return 0;
	}else if(c == 0xe1) {		//gV[PX E1.
		prefix |= PREFIX_E1;
		return 0;
	}else if(c == 0xf0) {		//ꂽ񂪌㑱.
		prefix |= PREFIX_F0;
		return 0;
	}

	// QoCgȏ̃V[PXȂAŌ̂PoCg:
	// ͒PL[̏ꍇɗ:

	prefix = 0;
	// [SHIFT] Ԃ̃tOXV.
	if((c == SKEY_LSHIFT)||(c == SKEY_RSHIFT)) {
		if(pre & PREFIX_F0) {
			kbd_shift &= KBD_SHIFT;
		}else{
			kbd_shift |= KBD_SHIFT;
		}
	}
	set_press_table(c,pre);

	if(pre & PREFIX_E0) {	//gV[PXSCAN CODEMSB𗧂Ăċʂ.
		c |= SKEY_EXTEND;
	}

	if(pre & PREFIX_F0) {	//ꂽ͖.
		return 0;
	}

	idx = c*2;
	if(kbd_shift & KBD_SHIFT) idx++;	// [SHIFT]Ă?

	// SCAN CODE --> ASCII CODE
	return __LPM( &keycode_table[idx] );
}
#endif
/**********************************************************************
 *	ps2kbd : 荞݃nh[.
 **********************************************************************
 */
INTERRUPT_SERVICE(PCINT1_vect)
{
	uchar edge = PS2KBD_PIN & PS2KBD_CLOCK_MASK;
											//	start and stop bits	are	ignored.
	if (!edge) {							// Routine entered at	falling	edge
		if(bitcount	< 11 &&	bitcount > 2) {	//	Bit	3 to 10	is data. Parity	bit,
			kbd_data >>= 1;
			if(PS2KBD_PIN & PS2KBD_DATA_MASK) {
				kbd_data |= 0x80;			// Store	a '1'
			}
		}
	} else {								// Routine entered at rising edge
		if(--bitcount == 0)	{				// All bits received
			put_que(kbd_data);
			bitcount = 11;
		}
	}
}

/**********************************************************************
 *	PCINT8 TIMER1 荞݂ G~[gipj.
 **********************************************************************
 */
#if	PS2KBD_USE_INTERRUPT
//  use PCINT8 hardhandler
#else
//	use TIMER1 softhandler
void 	kbd_int_handler(void)
{
	static uchar clk_last = PS2KBD_CLOCK_MASK;	// l = H

	uchar clk_edge = PS2KBD_PIN & PS2KBD_CLOCK_MASK;	//ݒl KBD_CLK

	// KBD_CLK̓dʂɕω?.(OƈقȂĂ邩)
	if(	clk_last != clk_edge ) {
		clk_last  = clk_edge;
		kbd_handler();					// PCINT8 ̃Gg[R[.
	}
}
#endif
/**********************************************************************
 *	
 **********************************************************************
 */
